import random
import numpy as np


def sampleList(percentage, maxValueList ):
    values = int(maxValueList * (percentage))
    extracted_List= random.sample(range(1, maxValueList+ 1), values)
    return extracted_List

def generateSubdf(original_df, train_df,column,percentage ):
    rowsToChange=len(original_df[column])*percentage
    dif = original_df[column] != train_df[column]
    diff_number = dif.sum()
    new_rowsToChange=rowsToChange-diff_number
    if new_rowsToChange<=0:
      newPercentage=0
      return train_df,newPercentage
    noise_df= train_df.copy()
    noise_df['id1'] = range(len(noise_df))
    or_df=original_df.copy()
    or_df['id1'] = range(len(or_df))
    mask = or_df[column] == noise_df[column]
    new_df = noise_df[mask]
    new_df = new_df.reset_index(drop=True)
    newPercentage=new_rowsToChange/len(new_df)
    return new_df,newPercentage

def mergeDataframe(noise_df,modified_df):

    merged_df = noise_df.merge(modified_df, on='id1', how='left', suffixes=('_df1', '_df2'))
    new_array = [string for string in noise_df.columns if string != 'id1']
    for col in new_array:
      noise_df[col] = merged_df[col + '_df2'].fillna(merged_df[col + '_df1'])
    noise_df = noise_df.drop('id1', axis=1)
    return noise_df



from pucktrick.utils import *
from pucktrick.binary import *
from pucktrick.categorical import *

def wrongLabelsBinaryExtended(original_df, train_df,column,percentage):
    noise_df= train_df.copy()
    noise_df['id1'] = range(len(noise_df))
    new_df,newPercentage=generateSubdf(original_df, noise_df,column,percentage)
    if newPercentage==0:
      return train_df
    modified_df= noiseBinaryNew(new_df,column,newPercentage)
    noise_df=mergeDataframe(noise_df,modified_df)
    noise_df[column] = noise_df[column].fillna(0)
    return noise_df

def wrongLabelsBinaryNew(train_df,target,percentage):
    noise_df=noiseBinaryNew(train_df,target,percentage)
    return noise_df

def  wrongLabelsCategoricalNew(train_df,target,percentage):
    noise_df=noiseCategoricalIntNewExistingValues(train_df,target,percentage)
    return noise_df

def wrongLabelsBinaryNew(train_df,target,percentage):
    noise_df=noiseCategoricalIntExtendedExistingValues(train_df,target,percentage)
    return noise_df



import unittest
from pucktrick.noisy import * 
import pandas as pd
from pandas.testing import assert_frame_equal
class TestCategorical(unittest.TestCase):
    def test_saluta(self):
        percentage=0.5
        num_rows = 1000
        fake_df=create_fake_table()
        column='f2'
        noise_df=noiseCategoricalStringNewFakeValues(fake_df,column,percentage)
        print(noise_df)
        dif = fake_df[column] != noise_df[column]
        diff_number = dif.sum()
        noise_df=noiseCategoricalStringExstendedFakeValues(fake_df,noise_df,column,percentage)
        dif = fake_df[column] != noise_df[column]
        diff_number = dif.sum()
        diff_p=diff_number/num_rows
    
        # Verifica che il numero di modifiche sia corretto
        self.assertEqual(diff_p, percentage)
    

def create_fake_table():
  num_rows = 1000
  # Generate data for each column
  f1 = np.random.uniform(-100, 100, num_rows)  # Continuous values between -100 and 100
  f2 = np.random.randint(-100, 101, num_rows)  # Discrete values between -100 and 100
  f3 = np.random.choice(['apple', 'banana', 'cherry'], num_rows)  # String values
  f4 = np.random.choice(['apple', 'banana', 'cherry'], num_rows)  # String values
  f5 = np.random.choice([0, 1], num_rows)  # Binary values (0 or 1)
  target = np.random.choice([0, 1], num_rows)  # Binary target (0 or 1)

  # Create the DataFrame
  df = pd.DataFrame({
    'f1': f1,
    'f2': f2,
    'f3': f3,
    'f4': f4,
    'f5': f5,
    'target': target
  })
  return df

if __name__ == "__main__":
    unittest.main()



# setup.py
from setuptools import setup, find_packages

setup(
    name="pucktrick",
    version="0.1",
    packages=find_packages(),
    install_requires=["numpy"],
    author="Andrea Maurino",
    author_email="andrea.maurino@unimib.it",
    description="A python library for error genration in dataset for machine learning",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/andreamaurino/pucktrick",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    license="CC BY-NC 4.0",
)